from typing import Optional, Union

from core.database.abc.db_model_dao_abc import DbModelDaoABC
from core.logger import DBLogger
from data.schemas.administration.user import User
from data.schemas.permission.permission_dao import permissionDao
from service.permission.permissions_enum import Permissions

logger = DBLogger(__name__)


class UserDao(DbModelDaoABC[User]):

    def __init__(self):
        DbModelDaoABC.__init__(self, __name__, User, "administration.users")

        self.attribute(User.keycloak_id, str)

    async def get_by_keycloak_id(self, keycloak_id: str) -> User:
        return await self.get_single_by({User.keycloak_id: keycloak_id})

    async def find_by_keycloak_id(self, keycloak_id: str) -> Optional[User]:
        return await self.find_single_by({User.keycloak_id: keycloak_id})

    async def has_permission(
        self, user_id: int, permission: Union[Permissions, str]
    ) -> bool:
        p = await permissionDao.get_by_name(
            permission if isinstance(permission, str) else permission.value
        )
        result = await self._db.select_map(
            f"""
            SELECT COUNT(*)
            FROM permission.role_users ru
            JOIN permission.role_permissions rp ON ru.roleId = rp.roleId
            WHERE ru.userId = {user_id} AND rp.permissionId = {p.id};
        """
        )
        if result is None or len(result) == 0:
            return False

        return True


userDao = UserDao()
